﻿using System;
using SpriteKit;
using CoreGraphics;
using UIKit;
using AVFoundation;
using Foundation;

namespace Color
{
	public class GameScene : SKScene
	{
		const int StageMax = 50;
		Block[] _red = new Block[StageMax];
		Block[] _green = new Block[StageMax];
		Block[] _blue = new Block[StageMax];
		int _stage = 1;
		int _hiScore = 5;
		int[] _rgb = new int[3];
		SKSpriteNode _titleSprite;
		SKSpriteNode _gameOverSprite;
		SKSpriteNode _stageClearSprite;
		SKLabelNode _scoreLabel;
		enum GameState { GS_TITLE, GS_MAIN, GS_GAMEOVER, GS_STAGECLEAR }
		GameState _gameState = GameState.GS_TITLE;
		AVAudioPlayer _bgmSound;
		AVAudioPlayer _gameStartSound;
		AVAudioPlayer _gameOverSound;
		AVAudioPlayer _stageClearSound;
		public AVAudioPlayer _mixSound;

		public GameScene(CGSize size)
		{
			Size = size;
			SetSprites();
			SetSounds();
			LoadHiScore();
			TitleScreen();
		}

		void SetSprites()
		{
			_titleSprite = SKSpriteNode.FromImageNamed("Title");
			_titleSprite.Position = new CGPoint(Size.Width / 2, Size.Height / 2);
			_gameOverSprite = SKSpriteNode.FromImageNamed("GameOver");
			_gameOverSprite.Position = new CGPoint(Size.Width / 2, Size.Height / 2);
			_stageClearSprite = SKSpriteNode.FromImageNamed("StageClear");
			_stageClearSprite.Position = new CGPoint(Size.Width / 2, Size.Height / 2);

			_scoreLabel = SKLabelNode.FromFont("HelveticaNeue-Medium");
			_scoreLabel.FontColor = UIColor.Black;
			_scoreLabel.FontSize = 30;
			_scoreLabel.Position = new CGPoint(Size.Width / 2, 10);
		}

		void SetSounds()
		{
			NSError err;
			NSUrl fileURL = new NSUrl("BGM.mp3");
			_bgmSound = new AVAudioPlayer(fileURL, "mp3", out err);
			_bgmSound.Play();
			_bgmSound.NumberOfLoops = -1;
			fileURL = new NSUrl("GameStart.mp3");
			_gameStartSound = new AVAudioPlayer(fileURL, "mp3", out err);
			fileURL = new NSUrl("GameOver.mp3");
			_gameOverSound = new AVAudioPlayer(fileURL, "mp3", out err);
			fileURL = new NSUrl("StageClear.mp3");
			_stageClearSound = new AVAudioPlayer(fileURL, "mp3", out err);
			fileURL = new NSUrl("Mix.mp3");
			_mixSound = new AVAudioPlayer(fileURL, "mp3", out err);
		}

		void LoadHiScore()
		{
			var prefs = NSUserDefaults.StandardUserDefaults;
			var score = prefs.IntForKey("HiScore");
			if (score > 0) _hiScore = (int)score;
		}

		void SaveHiScore()
		{
			if (_stage > _hiScore)
			{
				_hiScore = _stage;
				var prefs = NSUserDefaults.StandardUserDefaults;
				prefs.SetInt(_hiScore, "HiScore");
				prefs.Synchronize();
			}
		}

		public void TitleScreen()
		{
			BackgroundColor = UIColor.White;
			_stage = 1;
			_gameState = GameState.GS_TITLE;
			AddChild(_titleSprite);
			_scoreLabel.Text = "HiScore " + _hiScore;
			AddChild(_scoreLabel);
		}

		public void GameOverScreen()
		{
			_gameState = GameState.GS_GAMEOVER;
			AddChild(_gameOverSprite);
			_gameOverSound.Play();
			_gameOverSprite.ZPosition = 10000;
			SaveHiScore();
		}

		public void StageClearScreen()
		{
			_gameState = GameState.GS_STAGECLEAR;
			AddChild(_stageClearSprite);
			_stageClearSound.Play();
			_stageClearSprite.ZPosition = 10000;
		}

		public void InitStage()
		{
			Random rnd = new Random();
			_rgb[0] = _rgb[1] = _rgb[2] = 0;
			while (_rgb[0] == _rgb[1])
			{
				_rgb[0] = rnd.Next() % (_stage + 1);
                	_rgb[1] = rnd.Next() % (_stage + 1);
               		_rgb[2] = rnd.Next() % (_stage + 1);
			}
			SetBackgroundColor(0,0,0);
			for (int i = 0; i < _stage; i++)
			{
				_red[i]   = new Block(this, "Red"  , rnd);
				_green[i] = new Block(this, "Green", rnd);
				_blue[i]  = new Block(this, "Blue" , rnd);
			}
			_gameState = GameState.GS_MAIN;
			_gameStartSound.Play();
		}

		public void SetBackgroundColor(int r,int g,int b)
		{
			_rgb[0] += r;
			_rgb[1] += g;
			_rgb[2] += b;
			int max = (_rgb[0] < _rgb[1]) ? ((_rgb[1] < _rgb[2]) ? 2 : 1) : ((_rgb[0] < _rgb[2]) ? 2 : 0);
			if (_rgb[max] != 0)
			{
				float i = 1.0f / (float)_rgb[max];
				BackgroundColor = UIColor.FromRGB(i*_rgb[0], i*_rgb[1], i*_rgb[2]);
			}
			if (_rgb[0] == _rgb[1] && _rgb[0] == _rgb[2])
			{
				StageClearScreen();
			}
			System.Console.WriteLine("{0},{1},{2}",_rgb[0],_rgb[1],_rgb[2]);
		}

		private void Clear()
		{
			for (int i = 0; i < _stage; i++)
			{
				if (_red[i]._visible)
					_red[i].RemoveFromParent();
				_red[i].Dispose();
				if (_green[i]._visible)
					_green[i].RemoveFromParent();
				_green[i].Dispose();
				if (_blue[i]._visible)
					_blue[i].RemoveFromParent();
				_blue[i].Dispose();
			}
		}

		public override void Update(double currentTime)
		{
			base.Update(currentTime);
			bool flg = false;

			switch (_gameState)
			{
				case GameState.GS_TITLE:
					break;
				case GameState.GS_MAIN:
					for (int i = 0; i < _stage; i++)
					{
						if (_red[i].Move()) flg = true;
						if (_green[i].Move()) flg = true;
						if (_blue[i].Move()) flg = true;
					}
					if (!flg)
					{
						GameOverScreen();
					}
					break;
				case GameState.GS_GAMEOVER:
					break;
				case GameState.GS_STAGECLEAR:
					break;
			}
		}

		public override void TouchesEnded(Foundation.NSSet touches, UIEvent evt)
		{
			base.TouchesEnded(touches, evt);

			switch ( _gameState )
			{
				case GameState.GS_TITLE:
					_titleSprite.RemoveFromParent();
					_scoreLabel.RemoveFromParent();
					InitStage();
					break;
				case GameState.GS_MAIN:
					break;
				case  GameState.GS_GAMEOVER:
					_gameOverSprite.RemoveFromParent();
					Clear();
					TitleScreen();
					break;
				case GameState.GS_STAGECLEAR:
					_stageClearSprite.RemoveFromParent();
					Clear();
					if (_stage < StageMax) _stage++;
					InitStage();
					break;
			}
		}
	}
}
